#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <nan.h>
#include "contrast.h"
#include "edges.h"
#include "globals.h"


void norm_dbl(double *from, double *to, int size, double max)
{
  int i;
  double factor=255.0/max;
  FILE *fa, *fb;

  for(i=0; i<size; i++)
    to[i] = factor>0.0 ? from[i]*factor : 0.0;
}

void d_to_c(double *from, unsigned char *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i] = from[i]>0.0 ? (unsigned char)from[i] :0;
}


double contrast(unsigned char **feature_map,  
		double **(*dbl_contrast),
		double **gaus_c, double **gaus_s,
		int width, 
		int radius_c, 
		int num_features)
{
  int size = width * width;
  int radius_s=radius_c*DOG_Ratio;
  unsigned char *char_map;
  double max_inten;
  int i;
  double max;

  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;

  (*dbl_contrast) = (double **) calloc(num_features,  sizeof(double *));
  for (i=0; i<num_features; i++)
  {
    (*dbl_contrast)[i] = (double *) calloc(size,  sizeof(double));
  }
  
  /* calculate values for gaussians */
  Calc_Gaus(gaus_c, radius_c, radius_c*Sigma_c);
  Calc_Gaus(gaus_s, radius_s, radius_s*Sigma_s);


  max=Feature_Contrast(width, radius_c, radius_s, gaus_c, gaus_s,
			feature_map, (*dbl_contrast), num_features);

  return max;

} /* contrast()*/


int combine_maps(double max, double *values, double ***o_contrast,
		  double ***i_contrast, int index, int size)
{
  int i, j;
  double tmax, factor;
  char f_name[20];
  double **contrast_feature;
  int start, end;
  int o_start=0, o_end;

  o_end = Num_Orientations;
  factor=255.0/max;

  printf("max=%g\tfactor=%g\n", max, factor);
  if(IsINF(factor))
  {
    fprintf(stderr, "ERROR: Insufficient contrast found to continue\n");
    fprintf(stdout, "ERROR: Insufficient contrast found to continue\n");
    return 0;
  }

  if(!(O_Contrast || I_Contrast))
  {
    fprintf(stderr, "ERROR: No contrast dimension\n");
    return 0;
  }

  if (O_Target < 180)
  {
    o_start = (float)(O_Target)/180 * Num_Orientations;
    o_end = o_start+1;
  }

  if(O_Contrast && I_Contrast)
  {
    for (i=0; i<size; i++)
    {
      tmax=0;

      for (j=0; j<2; j++)
        if (i_contrast[index][j][i] > tmax)
  	  tmax=i_contrast[index][j][i];

      for (j=o_start; j<o_end; j++)
        if (o_contrast[index][j][i] > tmax)
  	  tmax=o_contrast[index][j][i];

      values[i] = (double) (tmax*factor);
    } /* for */
  } /* if */
  else 
  {
    if(O_Contrast) 
    {
      contrast_feature=o_contrast[index];
      start = o_start;
      end = o_end;
    }
    else 
    {
      contrast_feature=i_contrast[index];
      start = 0;
      end = 2;
    }

    printf("start = %d, end = %d\n", start, end);
    for (i=0; i<size; i++)
    {
      tmax=0;

      for (j=start; j<end; j++)
        if (contrast_feature[j][i] > tmax)
  	  tmax=contrast_feature[j][i];

      values[i] = (double) (tmax*factor);
    } /* for */
  } /* else */

  return 1;
} /* combine_maps */


double Feature_Contrast(int width, int center_radius, int radius, 
		   double **gaus_c, double **gaus_s,
		   unsigned char **edges, double **contrast_map,
		   int num_features)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  float weight[num_features];
  double cell[num_features];
  double max=0.0;

  Calc_Weights(weight, num_features);

  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<num_features; i++)
      {
        cell[i]=WEIGHT*Convolve(pixel_x, pixel_y, width,
                         center_radius, gaus_c, edges[i]);
        cell[i]=cell[i]-Convolve(pixel_x, pixel_y, width, radius,
                                 gaus_s, edges[i]);
      }

      for(i=0; i<num_features; i++)
      {
        for(j=0; j<num_features; j++)
	{
          contrast_map[i][pixel_y*width+pixel_x] 
			+= weight[(j-i+num_features)%num_features]
			   * cell[j];
	}
        if (contrast_map[i][pixel_y*width+pixel_x]>max)
	  max=contrast_map[i][pixel_y*width+pixel_x];
      } /* for (i) */

    }/* for (pixels) */

  return max;

}/* Feature_Contrast()*/


void Calc_Gaus(double **gaus, int rad, double sigma)
{
  int i,j;
  double total=0.0;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/(2.0*sigma));
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
}/* Calc_Gaus() */

void Calc_Weights(float *w, int num_features)
{
  int i;
  float denom;

  w[0]=1.0;
  w[1]=1.0;

  if (num_features%2)
    denom=num_features*num_features*num_features+2*num_features;

  for (i=1; i<=num_features/2; i++)
  {
    w[i]=((float)(-12*i*i))/denom;
    w[num_features-i]=w[i];
  }
/*
  denom=num_features*num_features;
  if (num_features%2)
    denom=denom-1;

  for (i=1; i<=num_features/2; i++)
  {
    w[i]=(float)(-4*i)/denom;
    w[num_features-i]=w[i];
  }
*/

} /* Calc_Weights() */


double Convolve(int x, int y, int width, int rad, double **gaus,
                unsigned char *values)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
        temp+=(double) values[(y-i)*width+(x-j)] *
                gaus[i+rad][j+rad];

  return temp;
} /* Convolve */














